

% Read EXCEL data: GNP and Unemployment 01/1950 -- 12/1987
[data,header] = xlsread('MacroData1.xls');
Date = header(2:end,1);
%------------
data = data(25:480,:);
Date = Date(25:480);
%----------------
GNP = data(:,1);
U = data(:,2);

% Compute GNP growth
mask = ~isnan(GNP);
grow = diff(log(GNP(mask))) * 100;
D_GNP = GNP(4:end);
mask = mask(4:end);
D_GNP(mask) = grow;

U = U(4:end);
Date = Date(4:end);
nobs = length(U);


% Follow Blanchard and Quah (1989), allow GNP growth has a
% structrual break at 01/1974
regime = 285;
Avg1 = mean(D_GNP(3:3:regime));
Avg2 = mean(D_GNP(regime+3:3:end));
D_GNP(1:regime) = D_GNP(1:regime) - Avg1;
D_GNP(regime+3:end) = D_GNP(regime+3:end) - Avg2;


% Follow Blanchard and Quah (1989), allow Umemployment has a linear trend
X = [ones(nobs,1),(1:nobs)'];
coeff = (X' * X) \ (X' * U);
U_detrend = U - X * coeff;

% Construct data matrix for VAR regression
Data_mix = [D_GNP,U_detrend];
U_detrend_mat = zeros(3,nobs/3);
U_detrend_mat(:) = U_detrend;
Data_quarter = [D_GNP(mask),mean(U_detrend_mat)'];

%---------------
% Quarterly VAR
%---------------
nlag = 8;
ndraws = 20000;
burn_in = ndraws * 0.5;
add_constant = 1;
[Phi_draws, Sigma_draws,constant_draws] = GIBBS_VAR(Data_quarter,nlag,ndraws,burn_in,add_constant);

% Recover structrual VAR and impulse response
constraint = 1;
MA_lag = 40;
shock_flag = [1,2];
response_flag = [1,2];
max_draws = 1000;
IRF1 = IRF_LONG(Phi_draws,Sigma_draws,constraint,MA_lag,shock_flag,response_flag,max_draws);

IRF1_sum = IRF1;
IRF1_sum(1,1,:,:) = cumsum(IRF1(1,1,:,:),3);
IRF1_sum(1,2,:,:) = cumsum(IRF1(1,2,:,:),3);

IRF1_mean = mean(IRF1_sum,4);
shock_size1 = IRF1_mean(:,:,1);
nshock = length(shock_flag);
nresponse = length(response_flag);
IRF1_lb = zeros(nresponse, nshock, MA_lag+1);
IRF1_ub = zeros(nresponse, nshock, MA_lag+1);
for m = 1:nshock
    for n = 1:nresponse
        for t = 1:MA_lag+1
           [IRF1_lb(n,m,t),IRF1_ub(n,m,t)] = HPD_SIM(IRF1_sum(n,m,t,:),0.05);
        end
    end
end


figure(1)
seq = (0:MA_lag)';
count = 1;
TXT_response = {'Output','Unemployment'};
TXT_shock = {'Demand','Supply'};
y_ub = [2, 0.5, 1.5, 0.5];
y_lb = [-1, -1, -1, -0.5];
for m = 1:nshock
    for n = 1: nresponse
        mean_use = IRF1_mean(n,m,:);
        ub_use = IRF1_ub(n,m,:);
        lb_use = IRF1_lb(n,m,:);
        % The blue solid line is the mean response
        % Two red dotted line is the 95% credible interval 
        subplot(nresponse,nshock,count),plot(seq,mean_use(:),'-b',  seq,ub_use(:),':r',  seq,lb_use(:),':r')
        title([TXT_response{response_flag(n)},' Response to ',TXT_shock{shock_flag(m)}])
        ylim([y_lb(count),y_ub(count)])
        count = count + 1;
    end
end

return

%---------------
% VARDAS
%---------------
ndraws = 20000;
burn_in = ndraws * 0.5;
nlag = 16;
MA_lag = MA_lag*3;
sum_flag = 1;
[Phi_draws, Sigma_draws,constant_draws] = VARDAS_DIFF(Data_mix,nlag,sum_flag,add_constant,ndraws,burn_in);
IRF2 = IRF_LONG(Phi_draws,Sigma_draws,constraint,MA_lag,shock_flag,response_flag,max_draws);
IRF2_sum = IRF2;
IRF2_sum(1,1,:,:) = cumsum(IRF2(1,1,:,:),3);
IRF2_sum(1,2,:,:) = cumsum(IRF2(1,2,:,:),3);

IRF2_mean = mean(IRF2_sum,4);
shock_size2 = IRF2_mean(:,:,1);
shock_size_ratio = shock_size1 ./ shock_size2;

nshock = length(shock_flag);
nresponse = length(response_flag);
IRF2_lb = zeros(nresponse, nshock, MA_lag+1);
IRF2_ub = zeros(nresponse, nshock, MA_lag+1);
for m = 1:nshock
    for n = 1:nresponse
        for t = 1:MA_lag+1
           [IRF2_lb(n,m,t),IRF2_ub(n,m,t)] = HPD_SIM(IRF2_sum(n,m,t,:),0.05);
        end
    end
end

figure(2)
seq = (0:MA_lag)'/3;
count = 1;
for m = 1:nshock
    for n = 1: nresponse
        
        % Normalize the size of the shock
        mean_use = IRF2_mean(n,m,:) * shock_size_ratio(n,m);
        ub_use = IRF2_ub(n,m,:) * shock_size_ratio(n,m);
        lb_use = IRF2_lb(n,m,:) * shock_size_ratio(n,m);
        
        % The blue solid line is the mean response
        % Two red dotted line is the 95% credible interval
        % Horizontal axis converted to quarters
        subplot(nresponse,nshock,count),plot(seq,mean_use(:),'-b',  seq,ub_use(:),':r',  seq,lb_use(:),':r')
        title([TXT_response{response_flag(n)},' Response to ',TXT_shock{shock_flag(m)}])
        ylim([y_lb(count),y_ub(count)])
        count = count + 1;
    end
end
