clear all
clc

% Read EXCEL data: GDP,CPI,FedFund,M1,  01/1959 -- 09/2011
[data,header] = xlsread('MacroData2.xls');
Date = header(2:end,1);
%--------------------
% Period 1974:01 to 2006:12
data = data(181:576,:);
Date = Date(181:576);
%--------------------

GDP = data(:,1);
CPI = data(:,2);
FedFund = data(:,3);
M1 = data(:,4);
nobs = length(CPI);

% Remove the trends by HP filter
CPI = HP_FILTER(CPI,14400)';
FedFund = HP_FILTER(FedFund,14400)';
M1 = HP_FILTER(M1,14400)';
GDP(3:3:end) = HP_FILTER(GDP(3:3:end),1600);
GDP = GDP';

% Rescale the data
GDP = GDP ./ 100;
M1 = M1 ./ 100;

% Plot the detrended data
figure(1)
seq = 1:nobs;
subplot(2,2,1),plot(seq,GDP);
subplot(2,2,2),plot(seq,CPI);
subplot(2,2,3),plot(seq,FedFund);
subplot(2,2,4),plot(seq,M1);

% Run quarterly data VAR
temp_mat = zeros(3,nobs/3);
temp_mat(:) = CPI; CPI_quarter = mean(temp_mat);
temp_mat(:) = FedFund; FedFund_quarter = mean(temp_mat);
temp_mat(:) = M1; M1_quarter = mean(temp_mat);
mask = ~isnan(GDP);
Data_quarter = [GDP(mask);CPI_quarter;FedFund_quarter;M1_quarter];
nlag = 4;
ndraws = 50000;
burn_in = ndraws*0.5;
add_constant = 1;
[Phi_draws, Sigma_draws,constant_draws] = GIBBS_VAR(Data_quarter,nlag,ndraws,burn_in,add_constant);
MA_lag = 36;
shock_flag = 3;
response_flag = 1:min(size(Data_quarter));
max_draws = 1000;
IRF1 = IRF_SHORT(Phi_draws,Sigma_draws,MA_lag,shock_flag,response_flag,max_draws);

% Draw impulse response graph
figure(2)
IRF1_mean = mean(IRF1,4);
shock_size1 = IRF1_mean(shock_flag,1,1);
nshock = length(shock_flag);
nresponse = length(response_flag);
IRF1_lb = zeros(nresponse, nshock, MA_lag+1);
IRF1_ub = zeros(nresponse, nshock, MA_lag+1);
for m = 1:nshock
    for n = 1:nresponse
        for t = 1:MA_lag+1
           [IRF1_lb(n,m,t),IRF1_ub(n,m,t)] = HPD_SIM(IRF1(n,m,t,:),0.05);
        end
    end
end
seq = (0:MA_lag)'*3;  % monthly response
count = 1;
TXT = {'GDP','CPI','Federal Funds Rate','M1'};
ylimit = [0.4, 0.3, 1.2, 0.07];
for m = 1:nshock
    for n = 1: nresponse
        mean_use = IRF1_mean(n,m,:);
        ub_use = IRF1_ub(n,m,:);
        lb_use = IRF1_lb(n,m,:);
        % The blue solid line is the mean response
        % Two red dotted line is the 95% credible interval
        % Horizontal axis converted to months
        subplot(2,2,count),plot(seq,mean_use(:),'-b',  seq,ub_use(:),':r',  seq,lb_use(:),':r')
        xlim([0,MA_lag])
        ylim([-ylimit(count),ylimit(count)])
        title([TXT{count},' Response'])        
        count = count + 1;
    end
end

% Run VARDAS regression
Data_mix = [GDP;CPI;FedFund;M1];
nlag = 6;
tic
[Phi_draws, Sigma_draws,constant_draws] = VARDAS_SIMPLE(Data_mix,nlag,1,add_constant,ndraws,burn_in);
toc
figure(3)
IRF2 = IRF_SHORT(Phi_draws,Sigma_draws,MA_lag,shock_flag,response_flag,max_draws);

% Draw impulse response graph
figure(3)
IRF2_mean = mean(IRF2,4);
shock_size2 = IRF2_mean(shock_flag,1,1);
shock_size_ratio = shock_size1 / shock_size2;
nshock = length(shock_flag);
nresponse = length(response_flag);
IRF2_lb = zeros(nresponse, nshock, MA_lag+1);
IRF2_ub = zeros(nresponse, nshock, MA_lag+1);
for m = 1:nshock
    for n = 1:nresponse
        for t = 1:MA_lag+1
           [IRF2_lb(n,m,t),IRF2_ub(n,m,t)] = HPD_SIM(IRF2(n,m,t,:),0.05);
        end
    end
end

seq = (0:MA_lag)';
nshock = length(shock_flag);
nresponse = length(response_flag);
count = 1;
for m = 1:nshock
    for n = 1: nresponse
        
        % Normalize the size of the shock
        mean_use = IRF2_mean(n,m,:) * shock_size_ratio;
        ub_use = IRF2_ub(n,m,:) * shock_size_ratio;
        lb_use = IRF2_lb(n,m,:) * shock_size_ratio;
        
        % The blue solid line is the mean response
        % Two red dotted line is the 95% credible interval
        subplot(2,2,count),plot(seq,mean_use(:),'-b',  seq,ub_use(:),':r',  seq,lb_use(:),':r')
        xlim([0,MA_lag])
        title([TXT{count},' Response'])
        ylim([-ylimit(count),ylimit(count)])
        count = count + 1;
    end
end
